/***********************************************************************
* A possible solution for MACRO based specializations of STD_VECTOR...
***********************************************************************/

#define EIGEN_STD_VECTOR_SPECIALIZATION_BODY_TPL(TRICKY_TYPE) \
  public:  \
  typedef TRICKY_TYPE value_type; \
  typedef typename vector_base::allocator_type allocator_type; \
  typedef typename vector_base::size_type size_type;  \
  typedef typename vector_base::iterator iterator;  \
  explicit vector(const allocator_type& a = allocator_type()) : vector_base(a) {}  \
  template<typename InputIterator> \
  vector(InputIterator first, InputIterator last, const allocator_type& a = allocator_type()) \
  : vector_base(first, last, a) {} \
  vector(const vector& c) : vector_base(c) {}  \
  explicit vector(size_type num, const value_type& val = value_type()) : vector_base(num, val) {} \
  vector(iterator start, iterator end) : vector_base(start, end) {}  \
  vector& operator=(const vector& x) {  \
  vector_base::operator=(x);  \
  return *this;  \
}

#if defined(_VECTOR_)
#define EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION_TPL(...) \
namespace std \
{ \
  template<typename _Ay> \
  class vector<__VA_ARGS__,_Ay>  \
    : public vector<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__), \
                    Eigen::aligned_allocator_indirection<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__)> > \
  { \
    typedef vector<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__), \
                   Eigen::aligned_allocator_indirection<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__)> > vector_base; \
    EIGEN_STD_VECTOR_SPECIALIZATION_BODY_TPL(__VA_ARGS__) \
    /* workaround MSVC std::vector implementation */ \
    void resize(size_type new_size) \
    { resize(new_size, __VA_ARGS__()); } \
    void resize(size_type new_size, const value_type& x) \
    { \
      if (vector_base::size() < new_size) \
        vector_base::_Insert_n(vector_base::end(), new_size - vector_base::size(), x); \
      else if (new_size < vector_base::size()) \
        vector_base::erase(vector_base::begin() + new_size, vector_base::end()); \
    } \
    void push_back(const value_type& x) \
    { vector_base::push_back(x); } \
    iterator insert(iterator position, const value_type& x) \
    { return vector_base::insert(position,x); } \
    iterator insert(iterator position, size_type new_size, const value_type& x) \
    { return vector_base::insert(position, new_size, x); } \
  }; \
}
#elif defined(_GLIBCXX_VECTOR) && EIGEN_GNUC_AT_LEAST(4,1) \
#define EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION_TPL(...) \
namespace std \
{ \
  template<typename _Ay> \
  class vector<__VA_ARGS__,_Ay>  \
    : public vector<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__), \
                    Eigen::aligned_allocator_indirection<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__)> > \
  { \
    typedef vector<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__), \
                   Eigen::aligned_allocator_indirection<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__)> > vector_base; \
    EIGEN_STD_VECTOR_SPECIALIZATION_BODY_TPL(__VA_ARGS__) \
    /* workaround GCC std::vector implementation */ \
    /* Note that before gcc-4.1 we already have: std::vector::resize(size_type,const T&), */ \
    /* no no need to workaround ! */ \
    void resize(size_type new_size) \
    { resize(new_size, __VA_ARGS__()); } \
    void resize(size_type new_size, const value_type& x) \
    { \
      if (new_size < vector_base::size()) \
        vector_base::_M_erase_at_end(this->_M_impl._M_start + new_size); \
      else \
        vector_base::insert(vector_base::end(), new_size - vector_base::size(), x); \
    } \
  }; \
}
#elif defined(_GLIBCXX_VECTOR)
#define EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION_TPL(...) \
namespace std \
{ \
  template<typename _Ay> \
  class vector<__VA_ARGS__,_Ay>  \
    : public vector<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__), \
                    Eigen::aligned_allocator_indirection<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__)> > \
  { \
    typedef vector<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__), \
                   Eigen::aligned_allocator_indirection<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__)> > vector_base; \
    EIGEN_STD_VECTOR_SPECIALIZATION_BODY_TPL(__VA_ARGS__) \
    void resize(size_type new_size) \
    { resize(new_size, __VA_ARGS__()); } \
    using vector_base::resize; \
  }; \
}
#else
#define EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION_TPL(...) \
namespace std \
{ \
  template<typename _Ay> \
  class vector<__VA_ARGS__,_Ay>  \
    : public vector<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__), \
                    Eigen::aligned_allocator_indirection<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__)> > \
  { \
    typedef vector<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__), \
                   Eigen::aligned_allocator_indirection<EIGEN_WORKAROUND_MSVC_STD_VECTOR(__VA_ARGS__)> > vector_base; \
    EIGEN_STD_VECTOR_SPECIALIZATION_BODY_TPL(__VA_ARGS__) \
    void resize(size_type new_size) \
    { resize(new_size, __VA_ARGS__()); } \
    /* default implementation which should always work. */ \
    void resize(size_type new_size, const value_type& x) \
    { \
      if (new_size < vector_base::size()) \
        vector_base::erase(vector_base::begin() + new_size, vector_base::end()); \
      else if (new_size > vector_base::size()) \
        vector_base::insert(vector_base::end(), new_size - vector_base::size(), x); \
    } \
  }; \
}
#endif
