#include <Eigen/Core>
#include <Eigen/StdVector>

#include "EigenMacros.h"

USING_PART_OF_NAMESPACE_EIGEN

/***********************************************************************
* Now the actual code...
***********************************************************************/

EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION_TPL(Vector2d)
EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION_TPL(Matrix2d)
EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION_TPL(Matrix<double, 2, 2, Eigen::AutoAlign | Eigen::RowMajor>)

struct Foo;
struct Bar;

EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION_TPL(Foo)
EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION_TPL(Bar)

struct Foo
{
  Foo() : v(5) {}
  std::vector<Vector2d> v;
};

struct Bar
{
  struct BarInternal
  {
    Vector2d v;
  };

  Bar() : bi(5) {}

  // I was not able to specialize for member structs/classes...
  std::vector<BarInternal, Eigen::aligned_allocator<BarInternal> > bi;
};

int main(int argc, char* argv[])
{
  {
    std::vector<Vector2d> std_vector;
    Vector2d v(1,2);
    std_vector.push_back(v);
    std_vector.resize(5);
    std_vector.resize(7,Vector2d(0,0));
  }

  {
    std::vector< Matrix2d > std_vector;
    Matrix2d v; v << 1,0,0,1;
    std_vector.push_back(v);
    std_vector.resize(5);
    std_vector.resize(7,Matrix2d::Zero(2,2));
  }

  {
    typedef Matrix<double, 2, 2, Eigen::AutoAlign | Eigen::ColMajor> RowMajorMatrix;

    std::vector<RowMajorMatrix> std_vector;
    Matrix2d v; v << 1,0,0,1;
    std_vector.push_back(v);
    std_vector.resize(5);
    std_vector.resize(7,RowMajorMatrix::Zero(2,2));
  }

  {
    std::vector<Foo> std_vector;
    std_vector.resize(5);
    std_vector.resize(7, Foo());
  }

  {
    std::vector<Bar> std_vector;
    std_vector.resize(5);
    std_vector.resize(7, Bar());
  }
}
